/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dscriptmodbus;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;

/**
 * This class creates a connection to a dScript module and has the nesseary methods in it to talk to it using modbus
 * 
 * @author James Henderson
 */
public class ModbusTCPConnection {

    private final int uid;
    private final int downstream_uid;
    private final Socket ds_socket;
    private final DataInputStream input;
    private final DataOutputStream output;
    
    /**
     * Try and create a connection to the module, if no connection can be made then an exception is thrown
     * 
     * @param ip the ip address of the module
     * @param port the port of the module
     * @param uid the modbus UID of the module
     * @throws IOException
     */
    public ModbusTCPConnection(String ip, int port, int uid, int downstream_uid) throws IOException {
        
        this.uid = uid;
        this.downstream_uid = downstream_uid;
        ds_socket = new Socket(ip, port);                 // Try and open the socket
        ds_socket.setSoTimeout(500);
        input = new DataInputStream(ds_socket.getInputStream());    // Get the input stream from the socket
        output = new DataOutputStream(ds_socket.getOutputStream()); // Get the output stream from the socket

    }
    
    
    /**
     * Set the state of a relay
     * 
     * @param relay
     * @param state
     * @throws IOException 
     */
    public void setRelayToState(byte relay, byte state) throws IOException {
        byte[] packet = {0, 0, 0, 0, 0, 6, (byte)uid, 5, 0, relay, state, 0};
        this.output.write(packet, 0, packet.length);
        byte[] responce = new byte[20];
        this.input.read(responce, 0, 20);
    }
    
    
    /**
     * Set the state of an io on the module
     * @param io
     * @param state
     * @throws IOException 
     */
    public void setIOToState(byte io, byte state) throws IOException {
        byte[] packet = {0, 0, 0, 0, 0, 6, (byte)uid, 5, 0, (byte)(40 + io), state, 0};
        this.output.write(packet, 0, packet.length);
        byte[] responce = new byte[20];
        this.input.read(responce, 0, 20);
    }
    
    
    /**
     * Set the state of a downstream relay on an attached MBH88
     * 
     * @param relay
     * @param state
     * @throws IOException 
     */
    public void setDownstreamRelayToState(byte relay, byte state) throws IOException {
        byte[] packet = {0, 0, 0, 0, 0, 6, (byte)downstream_uid, 5, 0, relay, state, 0};
        this.output.write(packet, 0, packet.length);
        byte[] responce = new byte[20];
        this.input.read(responce, 0, 20);
    }
    
    
    /**
     * Gets the states of the relays and inputs
     * 
     * @return
     * @throws IOException 
     */
    public byte[] getStates(byte points) throws IOException {
        // 1242 44 points
        // 378 47 points
        byte[] packet = { 0, 0, 0, 0, 0, 6, 1, 1, 0, 0, 0, points};
        this.output.write(packet, 0, packet.length);
        byte[] states = new byte[20];
        this.input.read(states, 0, 20);
        return states;
    }
    
    
    /**
     * Gets the analogue input values
     * 
     * @return
     * @throws IOException 
     */
    public byte[] getADCValues(byte points) throws IOException {
        //1242 24 points
        byte[] packet = {0, 0, 0, 0, 0, 6, (byte)uid, 4, 0, 0, 0, points};
        this.output.write(packet, 0, packet.length);
        byte[] responce = new byte[58];
        this.input.read(responce, 0, 58);
        return responce;
    }
    
    
    /**
     * gets the states of the downstream relays of a connected MBH88
     * 
     * @return
     * @throws IOException 
     */
    public byte[] getStateOfDownstreamRelays() throws IOException {
        byte[] packet = { 0, 0, 0, 0, 0, 6, (byte)downstream_uid, 1, 0, 1, 0, 8 };
        this.output.write(packet, 0, packet.length);
        byte[] responce = new byte[58];
        this.input.read(responce, 0, 20);
        return responce;
    }
    
    
    /**
     * Close the connection
     * 
     * @throws IOException 
     */
    public void close() throws IOException {
        this.input.close();
        this.output.close();
        this.ds_socket.close();
    }
    
    
    
}
